% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OUTmethod_SW.R
\name{OUTmethod_SW}
\alias{OUTmethod_SW}
\title{Fitting potential outcome regression with different methods in survey observational data}
\usage{
OUTmethod_SW(
  out.formula = out.formula,
  out.weights = NULL,
  y = y,
  out.method = "glm",
  family = "gaussian",
  datain = datain,
  dataout = dataout,
  out.control = list()
)
}
\arguments{
\item{out.formula}{an object of class \code{\link{formula}} (or one that can be coerced to that class):
a symbolic description of the outcome model to be fitted.}

\item{out.weights}{A numeric vector of weights to be used in the weighted regression estimator. 
For the moment estimator(MOM) and clever covariates estimator(CVR), this parameter defaults to NULL; 
however, in the weighted regression estimator(WET) it should be set to the balancing weights used for the weighted outcome regression model.}

\item{y}{a vector of the observed outcome in the training data (\code{datain}).}

\item{out.method}{a character to specify the method for estimating the outcome regression model. \code{"glm"} is default, and \code{"gbm"} and \code{"SuperLearner"} are also allowed.}

\item{family}{a description of the error distribution and link function to be used in the outcome model. Supported distributional families include
\code{"gaussian" (link = identity)}, \code{"binomial" (link = logit)} and \code{"poisson" (link = log)}. Default is \code{"gaussian"}.}

\item{datain}{The training data for the outcome model. In the context of \code{PSweight}, it refers to the data observed for each treatment group.}

\item{dataout}{The prediction data for the outcome model. In the context of \code{PSweight}, it refers to the full data.}

\item{out.control}{a list to specify additional options when \code{out.method} is set to \code{"gbm"} or \code{"SuperLearner"}.}
}
\value{
\describe{

\item{\code{ m.est}}{a vector of predicted outcome on the \code{dataout}.}

\item{\code{ gamma.h}}{estimated coefficient of the outcome model when \code{method = "glm"}.}

}
}
\description{
The function \code{OUTmethod_SW} is an internal function to estimate the potential outcomes given a specified model through formula.
It is built into function \code{PSweight}, and is used for constructing the augmented estimators.
}
\details{
A typical form for \code{out.formula} is \code{y ~ terms} where \code{y} is the outcome
variable and \code{terms} is a series of terms which specifies linear predictors (on the link function scale). \code{out.formula} by default specifies generalized
linear model given the gaussian error through the default arguments \code{method = "glm"} and \code{family = "gaussian"}.  It fits the logistic regression when \code{family = "binomal"},and poisson
regression when \code{family = "poisson"}. The argument \code{out.method} allows user to choose
model other than glm to fit the outcome regression models for constructing the augmented estimator. We have included \code{gbm} and \code{SuperLearner} as alternative machine learning estimators.
Additional argument in them can be supplied through the \code{...} argument. Please refer to the user manual of the \code{gbm} and \code{SuperLearner} packages for all the
allowed arguments.

When \code{out.weights} is provided, weighted regression is applied in all supported methods. In \code{"glm"}, 
weighted least squares is used for Gaussian outcomes, and weighted likelihood for binomial or Poisson outcomes. 
If \code{family = "poisson"} and non-integer outcomes are detected (due to continuous-valued responses or weighting effects), 
the function automatically switches to \code{"gaussian"} to ensure numerical stability. In \code{"gbm"}, outcome 
weights are passed via the \code{weights} argument, and if non-integer outcomes exist under \code{family = "poisson"}, 
the model switches to \code{"gaussian"}. For \code{"SuperLearner"}, weighted estimation is supported through the 
\code{obsWeights} argument. Since \code{"SuperLearner"} does not support Poisson regression, a warning is issued, 
and the model defaults to \code{"gaussian"}. These adjustments ensure compatibility and stability across different 
estimation approaches.
}
\examples{
# Load example datasets
data("psdata")
data("psdata_bin_prospective_fp")
data("psdata_bin_retrospective_fp")

# Define the outcome model formula.
out.formula <- Y ~ cov1 + cov2 + cov3 + cov4 + cov5 + cov6

# Extract the outcome vector from the retrospective data.
y <- psdata_bin_retrospective_fp$Y

# Use only the observations in treatment group 1 as the training data.
datain <- psdata_bin_retrospective_fp[psdata_bin_retrospective_fp$trt == 1, ]

# Fit the outcome regression model via OUTmethod_SW.
# By default, out.method = "glm" and family = "gaussian" are used.
outfit <- OUTmethod_SW(out.formula = out.formula, y = y, datain = datain,
                       dataout = psdata_bin_retrospective_fp)

# Print the predicted outcome vector on dataout.
cat("Predicted outcomes (first 10 values):\n")
print(head(outfit$m.est, 10))

# Print the estimated coefficient vector from the GLM.
cat("\nEstimated coefficients (gamma.h):\n")
print(outfit$gamma.h)

}
