% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PSmethod_SW.R
\name{PSmethod_SW}
\alias{PSmethod_SW}
\title{Fitting propensity score models with different methods in survey observational data}
\usage{
PSmethod_SW(
  ps.formula = ps.formula,
  method = "glm",
  data = data,
  survey.weight = survey.weight,
  ncate = ncate,
  ps.control = list()
)
}
\arguments{
\item{ps.formula}{an object of class \code{\link{formula}} (or one that can be coerced to that class):
a symbolic description of the propensity score model to be fitted. Additional details of model specification
are given under "Details". This argument is optional if \code{ps.estimate} is not \code{NULL}.}

\item{method}{a character to specify the method for estimating propensity scores. \code{"glm"} is default, and \code{"gbm"} and \code{"SuperLearner"} are also allowed.}

\item{data}{an optional data frame containing the variables in the propensity score model.}

\item{survey.weight}{an numeric vector specifying survey weights for each observation.}

\item{ncate}{a numeric to specify the number of treatment groups present in the given data.}

\item{ps.control}{a list to specify additional options when \code{method} is set to \code{"gbm"} or \code{"SuperLearner"}.}
}
\value{
\describe{

\item{\code{ e.h}}{a data frame of estimated propensity scores.}

\item{\code{ ps.fitObjects}}{the fitted propensity model details}

\item{\code{ beta.h}}{estimated coefficient of the propensity model when \code{method = "glm"}.}

}
}
\description{
The function \code{PSmethod_SW} is an internal function to estimate the propensity scores given a specified model through formula.
It is built into function \code{Sumstat_SW}, \code{PStrim_SW} and \code{PSweight_SW}.
}
\details{
A typical form for \code{ps.formula} is \code{treatment ~ terms} where \code{treatment} is the treatment
variable and \code{terms} is a series of terms which specifies a linear predictor. \code{ps.formula} by default specifies generalized
linear models given the default argument \code{method = "glm"}.  It fits the logistic regression when \code{ncate = 2},and multinomial
logistic regression when \code{ncate > 2}. The argument \code{method} allows user to choose
model other than glm to fit the propensity score models. We have included \code{gbm} and \code{SuperLearner} as two alternative machine learning methods.
Additional arguments of the machine learning estimators can be supplied through the \code{...} argument. Note that SuperLearner does not handle multiple groups and the current version of multinomial
logistic regression is not supported by gbm. We suggest user to use them with extra caution. Please refer to the user manual of the \code{gbm} and \code{SuperLearner} packages for all the
allowed arguments.
}
\examples{
# Load example datasets
data("psdata")
data("psdata_bin_prospective_fp")
data("psdata_bin_retrospective_fp")

# Define the propensity score model.
ps.formula <- trt ~ cov1 + cov2 + cov3 + cov4 + cov5 + cov6

# Extract the survey weights from the retrospective data.
survey.weight <- psdata_bin_retrospective_fp$survey_weight

# Specify the number of treatment groups (for binary treatment, ncate = 2)
ncate <- 2

# Fit the propensity score model using PSmethod_SW.
psfit <- PSmethod_SW(ps.formula = ps.formula,
                     data = psdata_bin_retrospective_fp,
                     survey.weight = survey.weight,
                     ncate = ncate)

# Print the first 10 rows of the estimated propensity scores.
cat("Estimated propensity scores (first 10 observations):\n")
print(head(psfit$e.h, 10))

# For the 'glm' method, print the estimated coefficient vector.
cat("\nEstimated coefficients (beta.h):\n")
print(psfit$beta.h)

# Users can also inspect the full fitted model object.
cat("\nFitted propensity model object summary:\n")
print(summary(psfit$ps.fitObjects))


}
